package ru.dbotthepony.kstarbound.io

import java.io.IOException
import java.io.InputStream
import java.io.RandomAccessFile

/**
 * Читает Variable Length Integer как Long
 */
fun RandomAccessFile.readVarLong(): Long {
	var result = 0L
	var read = read()

	while (true) {
		result = (result shl 7) or (read.toLong() and 0x7FL)

		if (read and 0x80 == 0) {
			break
		}

		read = read()
	}

	return result
}

/**
 * Читает Variable Length Integer как Int
 */
fun RandomAccessFile.readVarInt(): Int {
	var result = 0
	var read = read()

	while (true) {
		result = (result shl 7) or (read and 0x7F)

		if (read and 0x80 == 0) {
			break
		}

		read = read()
	}

	return result
}

fun RandomAccessFile.readASCIIString(length: Int): String {
	require(length >= 0) { "Invalid length $length" }

	val bytes = ByteArray(length)
	try {
		readFully(bytes)
	} catch(err: Throwable) {
		throw IOException("Tried to read string with length of $length", err)
	}

	return bytes.toString(Charsets.UTF_8)
}

fun InputStream.readASCIIString(length: Int): String {
	require(length >= 0) { "Invalid length $length" }

	val bytes = ByteArray(length)

	try {
		val read = read(bytes)
		require(read == bytes.size) { "Read $read, expected ${bytes.size}" }
	} catch(err: Throwable) {
		throw IOException("Tried to read string with length of $length", err)
	}

	return bytes.toString(Charsets.UTF_8)
}
