package ru.dbotthepony.kstarbound.test

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.kstarbound.math.*
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE_FF
import ru.dbotthepony.kstarbound.world.CHUNK_SIZEd
import ru.dbotthepony.kstarbound.world.ChunkPos

object MathTests {
	@Test
	@DisplayName("roundByAbsoluteValue test")
	fun roundByAbsoluteValueTest() {
		check(roundByAbsoluteValue(0.0) == 0)
		check(roundByAbsoluteValue(0.1) == 1)
		check(roundByAbsoluteValue(1.1) == 2)
		check(roundByAbsoluteValue(-0.1) == -1)
		check(roundByAbsoluteValue(-0.0) == 0)
		check(roundByAbsoluteValue(-1.0) == -1)
		check(roundByAbsoluteValue(-1.1) == -2)
	}

	@Test
	@DisplayName("roundTowardsNegativeInfinity test")
	fun roundTowardsNegativeInfinityTest() {
		check(roundTowardsNegativeInfinity(0.0) == 0)
		check(roundTowardsNegativeInfinity(0.1) == 0)
		check(roundTowardsNegativeInfinity(0.6) == 0)
		check(roundTowardsNegativeInfinity(1.1) == 1)
		check(roundTowardsNegativeInfinity(-0.1) == -1)
		check(roundTowardsNegativeInfinity(-0.0) == 0)
		check(roundTowardsNegativeInfinity(-1.0) == -1)
		check(roundTowardsNegativeInfinity(-1.1) == -2)
		check(roundTowardsNegativeInfinity(-1.6) == -2)
	}

	@Test
	@DisplayName("roundTowardsPositiveInfinity test")
	fun roundTowardsPositiveInfinityTest() {
		check(roundTowardsPositiveInfinity(0.0) == 0)
		check(roundTowardsPositiveInfinity(0.1) == 1)
		check(roundTowardsPositiveInfinity(0.6) == 1)
		check(roundTowardsPositiveInfinity(1.1) == 2)
		check(roundTowardsPositiveInfinity(-0.1) == 0)
		check(roundTowardsPositiveInfinity(-0.0) == 0)
		check(roundTowardsPositiveInfinity(-1.0) == -1)
		check(roundTowardsPositiveInfinity(-1.1) == -1)
		check(roundTowardsPositiveInfinity(-1.6) == -1)
	}

	@Test
	@DisplayName("AABB Basic Math")
	fun basicAABB() {
		val a = AABB.rectangle(Vector2d.ZERO, 1.0, 1.0)

		check(a.intersect(AABB.rectangle(Vector2d(-1.0), 1.0, 1.0)))
		check(!a.intersectWeak(AABB.rectangle(Vector2d(-1.0), 1.0, 1.0)))
		check(!a.intersect(AABB.rectangle(Vector2d(-2.0), 1.0, 1.0)))
		check(!a.intersectWeak(AABB.rectangle(Vector2d(-2.0), 1.0, 1.0)))

		check(a.intersect(AABB.rectangle(Vector2d(-0.9), 1.0, 1.0)))
		check(a.intersectWeak(AABB.rectangle(Vector2d(-0.9), 1.0, 1.0)))

		val bigA = AABB.rectangle(Vector2d.ZERO, 200.0, 200.0)
		val smallB = AABB.rectangle(Vector2d.ZERO, 1.0, 1.0)

		check(bigA.intersect(smallB))
		check(smallB.intersect(bigA))

		check(bigA.intersectWeak(smallB))
		check(smallB.intersectWeak(bigA))

		check(AABB.rectangle(Vector2d.ZERO, 1.0, 1.0) == AABB(Vector2d(-0.5, -0.5), Vector2d(0.5, 0.5)))

		val combineA = AABB(Vector2d(0.0, 0.0), Vector2d(2.0, 2.0))
		val combineB = AABB(Vector2d(2.0, 5.0), Vector2d(4.0, 6.0))

		check(combineA.combine(combineB) == AABB(Vector2d(0.0, 0.0), Vector2d(4.0, 6.0))) { combineA.combine(combineB).toString() }
	}

	@Test
	@DisplayName("AABB transformation to AABBi")
	fun transformAABB() {
		check(AABB.rectangle(Vector2d.ZERO, 2.0, 2.0).encasingIntAABB() == AABBi(Vector2i(-1, -1), Vector2i(1, 1)))
		check(AABB.rectangle(Vector2d.ZERO, 2.0, 2.0).encasingChunkPosAABB() == AABBi(Vector2i(-1, -1), Vector2i(0, 0))) { AABB.rectangle(Vector2d.ZERO, 2.0, 2.0).encasingChunkPosAABB()}

		check(AABB.rectangle(Vector2d(x = CHUNK_SIZEd - 0.1), 2.0, 2.0).encasingChunkPosAABB() == AABBi(Vector2i(0, -1), Vector2i(1, 0))) { "${AABB.rectangle(Vector2d(x = CHUNK_SIZEd - 0.1), 2.0, 2.0)} ${AABB.rectangle(Vector2d(x = CHUNK_SIZEd - 0.1), 2.0, 2.0).encasingChunkPosAABB()}" }
		check(AABB.rectangle(Vector2d(x = CHUNK_SIZEd + 0.1), 2.0, 2.0).encasingChunkPosAABB() == AABBi(Vector2i(0, -1), Vector2i(1, 0))) { "${AABB.rectangle(Vector2d(x = CHUNK_SIZEd + 0.1), 2.0, 2.0)} ${AABB.rectangle(Vector2d(x = CHUNK_SIZEd + 0.1), 2.0, 2.0).encasingChunkPosAABB()}" }
	}

	@Test
	@DisplayName("ChunkPos class tests")
	fun chunkPosTests() {
		check(ChunkPos.fromTilePosition(0, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(1, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(0, 1) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(1, 1) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(-1, 1) == ChunkPos(-1, 0))
		check(ChunkPos.fromTilePosition(-1, -1) == ChunkPos(-1, -1))

		check(ChunkPos.fromTilePosition(CHUNK_SIZE_FF, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(CHUNK_SIZE, 0) == ChunkPos(1, 0))
		check(ChunkPos.fromTilePosition(0, CHUNK_SIZE_FF) == ChunkPos(0, 0))
		check(ChunkPos.fromTilePosition(0, CHUNK_SIZE) == ChunkPos(0, 1))

		check(ChunkPos.fromTilePosition(-CHUNK_SIZE_FF, 0) == ChunkPos(-1, 0)) { ChunkPos.fromTilePosition(-CHUNK_SIZE_FF, 0).toString() }
		check(ChunkPos.fromTilePosition(-CHUNK_SIZE, 0) == ChunkPos(-2, 0)) { ChunkPos.fromTilePosition(-CHUNK_SIZE, 0) }
		check(ChunkPos.fromTilePosition(0, -CHUNK_SIZE_FF) == ChunkPos(0, -1)) { ChunkPos.fromTilePosition(0, -CHUNK_SIZE_FF) }
		check(ChunkPos.fromTilePosition(0, -CHUNK_SIZE) == ChunkPos(0, -2)) { ChunkPos.fromTilePosition(0, -CHUNK_SIZE) }
	}
}
